[clang-tools-extra] 100ec80 - [clang-tidy] Add early exit for defaulted FunctionDecls
Kirill Bobyrev via cfe-commits
cfe-commits at lists.llvm.org
Thu Feb 10 11:29:35 PST 2022
Author: Kirill Bobyrev
Date: 2022-02-10T20:29:26+01:00
New Revision: 100ec80ab5a6a01071771b67fc2141640d0984da
URL: https://github.com/llvm/llvm-project/commit/100ec80ab5a6a01071771b67fc2141640d0984da
DIFF: https://github.com/llvm/llvm-project/commit/100ec80ab5a6a01071771b67fc2141640d0984da.diff
LOG: [clang-tidy] Add early exit for defaulted FunctionDecls
This prevents matching of defaulted comparison operators.
Resolves: https://github.com/llvm/llvm-project/issues/53355
Author: Febbe (Fabian Keßler)
Added:
Modified:
clang-tools-extra/clang-tidy/modernize/UseTrailingReturnTypeCheck.cpp
clang-tools-extra/test/clang-tidy/checkers/modernize-use-trailing-return-type-cxx20.cpp
Removed:
################################################################################
diff --git a/clang-tools-extra/test/clang-tidy/checkers/modernize-use-trailing-return-type-cxx20.cpp b/clang-tools-extra/test/clang-tidy/checkers/modernize-use-trailing-return-type-cxx20.cpp
index de2198437e101..63fe7a95fdc94 100644
--- a/clang-tools-extra/test/clang-tidy/checkers/modernize-use-trailing-return-type-cxx20.cpp
+++ b/clang-tools-extra/test/clang-tidy/checkers/modernize-use-trailing-return-type-cxx20.cpp
@@ -52,3 +52,49 @@ template <typename T>
T req2(T t) requires requires { t + t; };
// CHECK-MESSAGES: :[[@LINE-1]]:3: warning: use a trailing return type for this function [modernize-use-trailing-return-type]
// CHECK-FIXES: {{^}}auto req2(T t) -> T requires requires { t + t; };{{$}}
+
+//
+// Operator c++20 defaulted comparison operators
+//
+// Requires <compare>
+
+namespace std {
+struct strong_ordering {
+ using value_type = signed char;
+ static strong_ordering const less;
+ static strong_ordering const equal;
+ static strong_ordering const equivalent;
+ static strong_ordering const greater;
+
+ constexpr strong_ordering(value_type v) : val(v) {}
+ template <typename T>
+ requires(T{0}) friend constexpr auto
+ operator==(strong_ordering v, T u) noexcept -> bool {
+ return v.val == u;
+ }
+ friend constexpr auto operator==(strong_ordering v, strong_ordering w) noexcept -> bool = default;
+
+ value_type val{};
+};
+inline constexpr strong_ordering strong_ordering::less{-1};
+inline constexpr strong_ordering strong_ordering::equal{0};
+inline constexpr strong_ordering strong_ordering::equivalent{0};
+inline constexpr strong_ordering strong_ordering::greater{1};
+
+} // namespace std
+
+struct TestDefaultOperatorA {
+ int a{};
+ int b{};
+
+ friend auto operator<=>(const TestDefaultOperatorA &, const TestDefaultOperatorA &) noexcept = default;
+};
+
+struct TestDefaultOperatorB {
+ int a{};
+ int b{};
+ friend auto operator==(const TestDefaultOperatorB &, const TestDefaultOperatorB &) noexcept -> bool = default;
+ friend bool operator<(const TestDefaultOperatorB &, const TestDefaultOperatorB &) noexcept = default;
+ // CHECK-MESSAGES: :[[@LINE-1]]:15: warning: use a trailing return type for this function [modernize-use-trailing-return-type]
+ // CHECK-FIXES: {{^}} friend auto operator<(const TestDefaultOperatorB &, const TestDefaultOperatorB &) noexcept -> bool = default;{{$}}
+};
More information about the cfe-commits
mailing list
diff --git a/clang-tools-extra/clang-tidy/modernize/UseTrailingReturnTypeCheck.cpp b/clang-tools-extra/clang-tidy/modernize/UseTrailingReturnTypeCheck.cpp
index 60b80af35c7d8..8d6322563f794 100644
--- a/clang-tools-extra/clang-tidy/modernize/UseTrailingReturnTypeCheck.cpp
+++ b/clang-tools-extra/clang-tidy/modernize/UseTrailingReturnTypeCheck.cpp
@@ -404,14 +404,17 @@ void UseTrailingReturnTypeCheck::check(const MatchFinder::MatchResult &Result) {
const auto *Fr = Result.Nodes.getNodeAs<FriendDecl>("Friend");
assert(F && "Matcher is expected to find only FunctionDecls");
- if (F->getLocation().isInvalid())
+ // Three-way comparison operator<=> is syntactic sugar and generates implicit
+ // nodes for all other operators.
+ if (F->getLocation().isInvalid() || F->isImplicit())
return;
// Skip functions which return just 'auto'.
const auto *AT = F->getDeclaredReturnType()->getAs<AutoType>();
- if (AT != nullptr && !AT->isConstrained() &&
- AT->getKeyword() == AutoTypeKeyword::Auto &&
- !hasAnyNestedLocalQualifiers(F->getDeclaredReturnType()))
+ if (AT != nullptr &&
+ ((!AT->isConstrained() && AT->getKeyword() == AutoTypeKeyword::Auto &&
+ !hasAnyNestedLocalQualifiers(F->getDeclaredReturnType())) ||
+ F->isDefaulted()))
return;
// TODO: implement those
@@ -404,14 +404,17 @@ void UseTrailingReturnTypeCheck::check(const MatchFinder::MatchResult &Result) {
return;
- // Skip functions which return just 'auto'.
+ // Skip functions which return 'auto' and defaulted operators.
const auto *AT = F->getDeclaredReturnType()->getAs<AutoType>();